/**
 * Created by estratek on 15/12/17.
 */
var React = require('react');
import GfsValidator, { getArrayErrors } from './GfsValidator';
import Select from 'react-select';
import { Select as SelectPlus } from 'react-select-plus';
var axios = require('axios');
import DynamicModal from './DynamicModal';
import GfsDescriptionPopOver from './GfsDescriptionPopOver';
import GfsOnlyView from './GfsOnlyView';
import { getUrlApi } from './utils';
import { BarLoader } from 'react-spinners';
import PropTypes from 'prop-types';
import createClass from 'create-react-class';

class DynamicSelect extends React.Component {
	state = {
		value: this.props.value,
		options: [],
		showModal: false,
		showModalEdit: false,
		showModalCreate: false,
		loadingData: false,
		errorLoading: false,
		readOnlyData: ''
	};
	canEdit(value) {
		return !!value && !(typeof value != 'object');
	}
	componentWillUpdate(nextProps) {
		if (nextProps.value !== this.props.value) {
			let newErrorState = getArrayErrors(nextProps.validation, nextProps.value).length > 0;
			this.props.onChangeError(newErrorState);
		}
	}
	componentDidUpdate = (prevProps, prevState) => {
		if (JSON.stringify(prevProps.paramsApi) != JSON.stringify(this.props.paramsApi)) {
			this.getData();
		}
	};

	componentDidMount() {
		if (this.props.readOnly) {
			return this.getReadOnlyData();
		}
		return this.getData();
	}
	getUrlAPI(_tipo) {
		return getUrlApi(_tipo, this.props);
	}
	getData() {
		if (this.props.options && Array.isArray(this.props.options)) return false;
		let _axios = this.props.axiosInstance || axios;
		this.setState({ loadingData: true });
		_axios
			.get(this.getUrlAPI(), { params: { ...this.removeNullParams(this.replaceDependencies(this.props.paramsApi)) } })
			.then(({ data }) => {
				this.setState({ options: data, loadingData: false });
			})
			.catch((err) => {
				this.setState({ errorLoading: true });
			});
	}
	removeNullParams(params) {
		let result = params;
		Object.keys(params).map(key => {
			if (params[key] == null || params[key] == "null"){
				delete result[key];
			}
		});
		return result;
	}
	getReadOnlyData() {
		this.setState({ loadingData: true });
		let _axios = this.props.axiosInstance || axios;
		// If other instance of axios is send it thru other Top Component
		_axios
			.get(`${this.getUrlAPI()}/${this.props.value}`)
			.then(({ data }) =>
				this.setState((prevState) => {
					return { readOnlyData: data, loadingData: false };
				})
			)
			.catch((err) => {
				this.setState({ errorLoading: true });
			});
	}
	handleChange(event) {
		if (!!event) {
			if (event instanceof Array) {
				let newArray = event.concat(event[this.props.valueKey]);
				this.props.onChange(event);
				this.setState({ value: event });
			} else {
				let newArray = event[this.props.valueKey];
				this.props.onChange(newArray, event);
				this.setState({ value: newArray });
			}
		} else {
			this.props.onChange(null);
			this.setState({ value: null });
		}
	}
	showModalCreate() {
		if (this.props.overWriteAdd) {
			return this.props.overWriteAdd();
		}
		this.setState({ showModalCreate: true });
	}
	onSave = (newData) => {
		this.setState(
			{
				options: this.state.options.concat(newData),
				showModalEdit: false,
				showModalCreate: false
			},
			() => this.handleChange(newData)
		);
	};
	onDelete(deleteData) {
		this.setState({
			showModalEdit: false,
			showModalCreate: false,
			options: this.state.options.filter((currentValue, index) => {
				return currentValue[this.props.valueKey] != deleteData[this.props.valueKey];
			})
		});
	}
	onUpdate(newData) {
		var newArray = [];
		var options = this.state.options;
		for (var value in options) {
			if (this.state.options[value][this.props.valueKey] != newData[this.props.valueKey]) {
				newArray[value] = this.state.options[value];
			} else {
				newArray[value] = newData;
			}
		}
		this.setState({ options: newArray });
	}
	showModalEdit() {
		if (!this.props.value) {
			return false;
		}
		this.setState({ showModalEdit: true });
	}

	hasError() {
		return getArrayErrors(this.props.validation, this.props.value).length > 0;
	}
	capitalizeLabelOnStructure = (str) => {
		let _str = {};
		Object.keys(str).map((el) => {
			_str = {
				..._str,
				[el]: {
					...str[el],
					label: str[el].label ? str[el].label.charAt(0).toUpperCase() + str[el].label.slice(1) : undefined
				}
			};
		});
		return _str;
	};
	valueRender = (_option) => {
		return <div>{this.getLabelKey(_option, this.props.labelKey)}</div>;
	};
	getLabelKey = (_option, _labelKey) => {
		let split = _labelKey.split(';');
		let value = '';
		split.map((_val) => {
			value += ' ' + _option[_val];
		});
		return value;
	};
	isNumber(n) {
		return !isNaN(parseFloat(n)) && isFinite(n);
	};
	isObjectId(id) {
		return /^[0-9a-fA-F]{24}$/.test(id);
	}
	getSelectedOption = (_value) => {
		// Verificar si es un array de ids
		try {
			let values = JSON.parse(_value);
			if(Array.isArray(values) && values.length > 0) {
				
				let options = this.props.options || this.state.options;
				return options.filter((option)=>{return values.includes(option[this.props.valueKey])})
			}
		} catch(e){}

		// Verificar si es solo un id
		if(typeof _value != 'object' && typeof _value != 'array'){
			let options = this.props.options || this.state.options;
			return (options.filter((obj)=>{return obj[this.props.valueKey]==_value}))[0];
		} else {
			return _value;
		}
	};
	getExpressionsInObj = (json) => {
		let jsonString = JSON.stringify(json);
		const patron = /\{\{([^{}]+)\}\}/g;
		const expresiones = jsonString.match(patron) || [];
		return expresiones;
	}
	replaceDependencies = (_propertyValue) => {
		if(!this.props.parentStructure){
			return _propertyValue;
		}
		// Las dependencias son referencias a otros campos y se puede agregar en cualquiera de las siguientes propiedades		
		const replaceExpression = (_expression) => {
			if(_expression.indexOf("{") < 0) return _expression;
			return this.props.parentStructure[_expression.replace(/{/g,"").replace(/}/g,"")].value || null;
		}
		// Para el valor remplazar valores de dependencias
		if(typeof _propertyValue === 'object' && Object.keys(_propertyValue).length>0){
			let jsonString = JSON.stringify(_propertyValue);
			const expresiones = this.getExpressionsInObj(_propertyValue);
			expresiones.map((expression)=>{
				jsonString = jsonString.replace(expression,replaceExpression(expression));
			});
			_propertyValue = JSON.parse(jsonString);
		}
		if(typeof _propertyValue != 'object' && typeof _propertyValue != 'array'){
			_propertyValue = replaceExpression(_propertyValue);
		}
		return _propertyValue;
	};
	render() {
		if (this.props.readOnly) {
			if (this.state.readOnlyData) {
				//si el this.state.readOnlyData[this.props.labelKey] es un arreglo de objetos se debe concatenar cada labelkey de cada objeto separandolo por comas
				if (Array.isArray(this.state.readOnlyData) && !!this.props.options) {
					let values = this.props.options;
					
					let stringValues = '';
					values.map((value, index) => {
						stringValues += value[this.props.labelKey] + (index < values.length - 1 ? ', ' : '');
					});
					return (
						<GfsOnlyView label={this.props.label} valueString={stringValues} />
					);
				}
				else{
					return (
						<GfsOnlyView label={this.props.label} valueString={this.state.readOnlyData[this.props.labelKey]} />
					);
				}
				
			}
			return (
				<div>
					{this.state.loadingData && <BarLoader height={1} width="100%" />}
					<GfsOnlyView label={this.props.label} />
				</div>
			);
		}
		return (
			<div
				id={`dselect_${this.props.model}_${this.props.microservice}`}
				className={
					(!this.props.classWrapp ? 'form-group' : this.props.classWrapp) +
					' ' +
					(this.hasError() ? 'has-error' : '')
				}
			>
				<DynamicModal
					isOpen={this.state.showModalCreate}
					model={this.props.model}
					onSave={this.onSave}
					microservice={this.props.microservice}
					onClose={() => this.setState({ showModalCreate: false })}
					host={this.props.host}
					title={this.props.label || this.props.model}
					axiosInstance={this.props.axiosInstance}
					transformStructure={this.capitalizeLabelOnStructure}
				/>
				<DynamicModal
					isOpen={this.state.showModalEdit}
					model={this.props.model}
					onSave={this.onSave}
					onUpdate={this.onUpdate.bind(this)}
					onDelete={this.onDelete.bind(this)}
					hideDeleteButton={this.props.hideDeleteButton}
					onClose={() => this.setState({ showModalEdit: false })}
					microservice={this.props.microservice}
					id={this.props.value}
					host={this.props.host}
					title={'Editar ' + this.props.label || this.props.model}
					axiosInstance={this.props.axiosInstance}
					transformStructure={this.capitalizeLabelOnStructure}
				/>

				{!!this.props.label && (
					<label className="form-control-label mb-1 ">
						{' '}
						<strong> {this.props.label}</strong>
						{this.props.description && <GfsDescriptionPopOver description={this.props.description} />}
					</label>
				)}
				<div className="input-button-Actions f-right">
					{!this.props.hideAddButton && (
						<button className="btn btn-link btn-sm" onClick={this.showModalCreate.bind(this)}>
							Agregar
						</button>
					)}
					{!this.props.hideUpdateButton &&
						(this.props.value && (
							<button
								className="btn btn-link btn-sm"
								onClick={this.showModalEdit.bind(this)}
								disabled={this.canEdit(this.props.value)}
							>
								Editar
							</button>
						))}
				</div>
				{this.props.hasGroups && (
					<SelectPlus
						disabled={this.props.disabled || this.state.loadingData || this.state.errorLoading}
						value={this.props.value}
						onChange={this.handleChange.bind(this)}
						labelKey={this.props.labelKey}
						valueKey={this.props.valueKey}
						placeholder={this.props.placeHolder}
						multi={this.props.multiple}
						options={this.props.options || this.state.options}
					/>
				)}
				{!this.props.hasGroups && (
					<Select
						id={`only_dselect_${this.props.model}_${this.props.microservice}`}
						disabled={this.props.disabled || this.state.loadingData || this.state.errorLoading}
						isDisabled={this.props.disabled || this.state.loadingData || this.state.errorLoading}
						value={this.getSelectedOption(this.props.value)}
						onChange={this.handleChange.bind(this)}
						getOptionLabel={(option) => option[this.props.labelKey]}
						getOptionValue={(option) => option[this.props.valueKey]}
						labelKey={this.props.labelKey}
						valueKey={this.props.valueKey}
						placeholder={this.props.placeHolder}
						multi={this.props.multiple}
						isMulti={this.props.multiple}
						isClearable	
						// valueRenderer={this.valueRender}
						// optionRenderer={this.valueRender}
						options={this.props.options || this.state.options}
						className={this.props.className}
						classNamePrefix={this.props.classNamePrefix}
					/>
				)}
				{this.state.loadingData && <BarLoader height={1} width="100%" />}
				{this.state.errorLoading && <div style={{ color: 'red' }}>Error obteniendo data</div>}

				<GfsValidator validation={this.props.validation} value={this.props.value} />
			</div>
		);
	}
}

DynamicSelect.defaultProps = {
	onChangeError: () => { },
	onChange: () => { },
	onChangeMultipleArrayKeys: () => { },
	value: '',
	paramsApi: {}
};

DynamicSelect.propTypes = {
	disabled: PropTypes.bool,
	type: PropTypes.string,
	validation: PropTypes.object,
	value: PropTypes.oneOfType([PropTypes.string, PropTypes.array]),
	paramsApi: PropTypes.object,
	onChange: PropTypes.func,
	onChangeError: PropTypes.func,
	hideUpdateButton: PropTypes.bool,
	hideAddButton: PropTypes.bool,
	model: PropTypes.string,
	host: PropTypes.string,
	microservice: PropTypes.string,
	rawOption: PropTypes.array,
	valueKey: PropTypes.string.isRequired,
	labelKey: PropTypes.string.isRequired,
	description: PropTypes.string,
	placeHolder: PropTypes.string,
	hasError: PropTypes.bool,
	classWrapp: PropTypes.string,
	label: PropTypes.string,
	onChangeMultipleArrayKeys: PropTypes.func,
	axiosInstance: PropTypes.instanceOf(axios),
	overWriteAdd: PropTypes.func
};
export default DynamicSelect;
