/**
 * Created by estratek on 15/12/17.
 */
//import React, {Component} from 'react';
//import {Modal}
//import Prueba from './Prueba';
import { shallow, mount } from 'enzyme';
import sinon from 'sinon';
import GenericForm from './GenericForm';
import React, { Component } from 'react';

describe('GenericForm', () => {
    describe('Renderizado', () => {
        it('debe renderizar todos los tipos de componentes', () => {
            let structure = {
                nombre: { type: 'text', value: 'hola' },
                numero: { type: 'number' },
                hola: { type: 'number' },

            }
            let comp = mount(<GenericForm structure={structure} />);
            expect(comp.find('input')).toHaveLength(3)
        });

        it('debe llamarse el OnChange en cada cambio de valor', () => {
            let structure = {
                nombre: { type: 'text', value: 'hola' },
                numero: { type: 'number' },
                hola: { type: 'number' }
            };
            let value = { nombre: 'rod' };
            let spy = sinon.spy();
            let comp = mount(<GenericForm
                structure={structure}
                value={value}
                onChange={spy} />);

            comp.find('input[type="text"]').simulate('change', { target: { value: 'testing' } });
            expect(spy.called).toBe(true);
            expect(spy.lastCall.args[0].form).toEqual({ nombre: 'testing' });
        });

        it('al modificarse el value debe renderizarse de nuevo con los nuevos valores', () => {
            let structure = { nombre: { type: 'text', value: 'before' }, };
            let comp = mount(<GenericForm
                structure={structure}
                value={{}} />);

            expect(comp.find('input[type="text"]').first().props().value).toEqual('before');
            comp.setProps({ value: { form: { nombre: 'after' } } });
            expect(comp.find('input[type="text"]').first().props().value).toEqual('after');
        });

        it('debe funcionar readOnly', () => {
            let structure = { 
                nombre: { type: 'text', value: 'before' }, 
                nombreVista:{type:'text',value:'vista'}
            };
            let comp = mount(<GenericForm
                structure={structure}
                value={{}} />);

            expect(comp.find('input[type="text"]')).toHaveLength(2);
            comp.setProps({structure:{...structure,nombreVista:{type:'text',value:'vista',readOnly:true}} });
            expect(comp.find('input[type="text"]')).toHaveLength(1);
        });
    })

    describe('ERRORES', () => {
        it('Verificar errores en Carga', () => {
            let spy = sinon.spy();
            let structure = {
                nombre: { type: 'text', value: 'Rodrigo' },
                apellido: { type: 'text', value: '', required: true },
            };
            let comp = mount(<GenericForm
                structure={structure}
                onChange={spy}
                value={{}} />);
            //onchange debe ser llamado con un argumento
            expect(spy.lastCall.args).toHaveLength(1);
            //Has error debe ser igual a verdadero
            expect(spy.lastCall.args[0].hasError).toBe(true);
        });

        it('Debe mostrar errores luego de ser cambiado un input', () => {
            let spy = sinon.spy();
            let structure = {
                nombre: { type: 'text', value: 'Rodrigo', required: true },
                apellido: { type: 'text', value: 'Chaclan' },
            };
            let comp = mount(<GenericForm
                structure={structure}
                onChange={spy}
                value={{}} />);

            //onchange debe ser llamado con un argumento
            expect(spy.lastCall.args).toHaveLength(1);
            //Has error debe ser falso ya que no hay errores
            expect(spy.lastCall.args[0].hasError).toBe(false);
            comp.find('input[type="text"]').first().simulate('change', { target: { value: '' } });
            //hasError debe ser verdadero
            expect(spy.lastCall.args[0].hasError).toBe(true);
        });
    })
});
