/**
 * Created by estratek on 15/12/17.
 */
import GfsValidator, { getArrayErrors } from './GfsValidator'
import PropTypes from 'prop-types';
import GfsOnlyView from './GfsOnlyView';
import InlineEdit from '@atlaskit/inline-edit';
import Textfield from '@atlaskit/textfield';
import GfsDescriptionPopOver from './GfsDescriptionPopOver'
var React = require('react');


class GfsInputAction extends React.Component {
    state={
        popoverOpen:false,
    }
    
    componentWillUpdate(nextProps) {
        if (nextProps.value !== this.props.value && !!this.props.onChangeError) {
            let newErrorState = getArrayErrors(nextProps.validation, nextProps.value).length > 0;
            this.props.onChangeError(newErrorState);
        }
    }
    componentDidMount() {   
        
        this.props.onChangeError(this.hasError());
    }
    handleChange = (event) => {
        
        let newVal = (this.props.type == 'checkbox') ? event.target.checked : event.target? event.target.value: event;
        
        this.props.onChange(newVal);
    }
    hasError = () => {
        return getArrayErrors(this.props.validation, this.props.value).length > 0
    }

    render() {
        if (this.props.readOnly) {
            return (<GfsOnlyView label={this.props.label} valueString={this.props.value} />)
        }
        return (
            
            <div className="mb-2 gfs-input gfs-input-text">                
                <div className={(!this.props.classWrapp ? "form-group input-group-dropdown mb-0 " : this.props.classWrapp) +
					' ' +
					(this.hasError() ? 'has-error' : '')}>
                    <label className="form-control-label mb-1"> <strong> {this.props.label}</strong>
                        {this.props.description && <GfsDescriptionPopOver description={this.props.description} />}
                    </label>

                    <div className="input-group" style={{marginBottom: "0px !important", marginTop: "0px !important"}}
>
                        {!!this.props.describerComponent && this.props.describerComponent}
                        
                        <InlineEdit
                            defaultValue={this.props.value|| ""}
                            
                            editView={({ errorMessage, ...fieldProps }) => (
                                <Textfield
                                    autocomplete="off"
                                    value={this.props.value}
                                    type={this.props.type}
                                    {...fieldProps}
                                    className={"form-control" + ((this.hasError()) ? 'form-control-danger' : '')}
                                    lang="en"
                                    checked={this.props.value}
                                    autoFocus
                                    placeholder={this.props.placeHolder || ''}
                                />
                            )}
                            readView={() => (
                                <React.Fragment>
                                    
                                    <input type={this.props.type}
                                        className={"form-control class" + ((this.hasError()) ? 'form-control-danger' : '')}
                                        value={this.props.value}
                                        checked={this.props.value}
                                        step={this.props.step}
                                        autocomplete="new-password"
                                        disabled={this.props.disabled}
                                        placeholder={this.props.placeHolder || ''} />
                                    
                                </React.Fragment>
                            )}
                            onConfirm={(value) => this.handleChange(value)}
                        />
                    </div>

                    <GfsValidator validation={this.props.validation}
                        value={this.props.value} />

                    
                </div>
            </div>
        );
    }
}




GfsInputAction.propTypes = {
    disabled: PropTypes.bool,
    type: PropTypes.string,
    validation: PropTypes.object,
    value: PropTypes.string.isRequired,
    onChange: PropTypes.func,
    onChangeError: PropTypes.func,
    description: PropTypes.string,
    placeHolder: PropTypes.string,
    hasError: PropTypes.bool,
    label: PropTypes.string,
    readOnly: PropTypes.bool
};
GfsInputAction.defaultProps = {
    disabled: false,
    type: 'text',
    validation: {},
    value: '',
    onChange: () => { },
    onChangeError: () => { },
    description: "",
    placeHolder: "",
    hasError: false,
    label: "",
    readOnly: false
};

export default GfsInputAction;