/**
 * Created by estratek on 15/12/17.
 */
var React = require('react');
import GfsValidator, { getArrayErrors } from './GfsValidator'
import DatePicker, { registerLocale } from 'react-datepicker';
var moment = require('moment');
import PropTypes from 'prop-types';
import Spanish from 'date-fns/locale/es';
import parseISO from 'date-fns/parseISO'
// registerLocale("es", Spanish);
import GfsOnlyView from './GfsOnlyView';
import { utc } from 'moment';


class GfsInputDatePicker extends React.Component {
    componentWillUpdate(nextProps) {
        if (nextProps.value !== this.props.value && !!this.props.onChangeError) {
            let newErrorState = getArrayErrors(nextProps.validation, nextProps.value).length > 0;
            this.props.onChangeError(newErrorState);
        }
    }
    componentDidMount = () => {
        registerLocale("es", Spanish);
    }

    handleChange = (event) => {
        // let newVal=(new Date(event)).toISOString();
        //console.log('newVal', event)
        this.props.onChange(event);
    }
    onChangeError(newState) {
        if (newState != this.state.e_hasError) {
            this.props.onChangeError(newState);
        }
    }
    prepareValue = (_value) => {
        return (!!_value) ? parseISO(_value.replace("000Z", "")) : null;
    }
    hasError = () => (
        !this.props.validation && getArrayErrors(this.props.validation, this.props.value).length > 0
    )

    waitForString = (stringDate) => {
        if (stringDate) {
            return new Date(stringDate);
        } else {
            return null;
        }
    }

    handleDateChangeRaw = (e) => {
        e.preventDefault();
    }
    render() {
        let endValue = this.props.endDate;
        if (this.props.readOnly) {
            return (<GfsOnlyView label={this.props.label} valueString={(!!this.props.value) ? moment(this.props.value).format('SSS') == '000' ? moment(this.props.value).locale('es').utc().format('DD MMM YYYY') : moment(this.props.value).locale('es').format('DD MMM YYYY') : ''} />)
        }
        return (
            <div className={"form-group " + ((this.hasError()) ? 'has-error' : '')}>
                {this.props.label && <label className="form-control-label"> <strong>{this.props.label}</strong></label>}
                <DatePicker
                    disabled={this.props.disabled}
                    locale="es"
                    className={"form-control " + ((this.hasError()) ? 'form-control-danger' : '')}
                    placeholderText={this.props.placeHolder}
                    selected={this.prepareValue(this.props.value)}
                    startDate={this.prepareValue(this.props.startDate)}
                    endDate={this.prepareValue(this.props.endDate)}
                    selectsStart={this.props.selectsStart || false}
                    selectsEnd={this.props.selectsEnd || false}
                    onChange={this.handleChange}
                    minDate={(this.props.startDate && (new Date(this.props.startDate))) || null}
                    maxDate={(this.props.endDate && (new Date(this.props.endDate))) || null}
                    isClearable={true}
                    onChangeRaw={this.handleDateChangeRaw}
                    peekNextMonth
                    showMonthDropdown
                    showYearDropdown
                    dropdownMode="select"
                    showMonthYearPicker={this.props.onlyMonth || false}
                    showFullMonthYearPicker={this.props.onlyMonth || false}
                    showTimeSelect={this.props.showTimeSelect}
                    timeFormat={this.props.showTimeSelect && "HH:mm"}
                    timeIntervals={this.props.timeIntervals}
                    timeCaption={this.props.timeCaption}
                    // dateFormatCalendar="MMMM"
                    fixedHeight
                    dateFormat={this.props.format || "dd/MM/yyyy"}
                    popperPlacement="top"
                // scrollableYearDropdown
                />
                <GfsValidator validation={this.props.validation}
                    value={this.props.value} />
                {this.props.description && <small id="emailHelp" className="form-text text-muted">{this.props.description}</small>}
            </div>
        );
    }
}
GfsInputDatePicker.propTypes = {
    disabled: PropTypes.bool,
    type: PropTypes.string,
    validation: PropTypes.object,
    value: PropTypes.string.isRequired,
    onChange: PropTypes.func,
    onChangeError: PropTypes.func,
    description: PropTypes.string,
    placeHolder: PropTypes.string,
    hasError: PropTypes.bool,
    label: PropTypes.string,
    readOnly: PropTypes.bool
};
GfsInputDatePicker.defaultProps = {
    disabled: false,
    type: 'text',
    validation: {},
    value: '',
    onChange: () => { },
    onChangeError: () => { },
    description: "",
    placeHolder: "",
    hasError: false,
    label: "",
    readOnly: false
};

export default GfsInputDatePicker;