
export default (axios) =>{
    const cacheable = true;
    var cache = require('js-cache');
    // On request, return the cached version, if any
    axios.interceptors.request.use(request => {

        // Solo cachar gets
        if (request.method === 'get' && cacheable) {
            let url = request.url;

            // Añadir parametros de url
            if (request.params)
                url += '?' + $.param(request.params);

            const _cached = cache.get(url);

            if (_cached) {
                // _cached.__fromCache = true;

                request.data = _cached;

                // Setear adapter a la request para enviar los cachado y prevenir llamada
                request.adapter = () => {
                    return Promise.resolve({
                        data: _cached,
                        status: request.status,
                        statusText: request.statusText,
                        headers: request.headers,
                        config: request,
                        request: request
                    });
                };
            }
        }

        return request;

    }, error => Promise.reject(error));

    // Interceptor de respuesta setear o borrar cache
    axios.interceptors.response.use(response => {

        // Validar parametro para omitir cache `__cache = false`
        const isCacheable = !response.config.params || (response.config.params && response.config.params.__cache !== false);

        if (cacheable && isCacheable) {
            let url = response.config.url;

            if (response.config.params)
                url += '?' + $.param(response.config.params);

            if (response.config.method === 'get') {
                // En la respuesta del get cachear respuesta
                cache.set(url, response.data);
            } else {
                // Para post, put or delete, borrar cache de la url get
                // e.g. post hacia `/users` borrara la cache `/users`, cuando se consulte '/users' no utilice data de cache
                cache.del(response.config.url);

                // Borrar cache de urls que tienen similitud
                // e.g. `/users?with=permissions`
                const urls = Object.keys(cache.debug());

                for (const _url of urls) {
                    if (_url.match(/^[^?]+/)[0] === response.config.url)
                        cache.del(_url);
                }
            }
        }

        return response;
    }, error => Promise.reject(error));
};